from django.conf import settings
from celery import shared_task
from piwikapi.tracking import PiwikTracker
from piwikapi.tests.request import FakeRequest
from urllib.parse import urlencode


class ExtendedPiwikTracker(PiwikTracker):
    """ We have to extend this to add the set_user_id """
    user_id = None

    def set_user_id(self, user_id: str):
        self.user_id = user_id

    def _get_request(self, id_site):
        url = super()._get_request(id_site)
        if self.user_id:
            url = url + '&' + urlencode({'uid': self.user_id})
        return url


@shared_task
def record_analytic(headers: dict, ip: str, user_id: str = None):
    """ Send analytics data to piwik """
    # Use "FakeRequest" because we had to serialize the real request
    request = FakeRequest(headers)
    pt = ExtendedPiwikTracker(settings.MATOMO_SITE_ID, request)
    pt.set_api_url(settings.MATOMO_API_URL)
    if settings.MATOMO_TOKEN_AUTH:
        pt.set_token_auth(settings.MATOMO_TOKEN_AUTH)
        pt.set_ip(ip)
    if getattr(settings, 'MATOMO_TRACK_USER_ID', None) and user_id:
        pt.set_user_id(user_id)
    # Truncate it
    visited_url = request.META['PATH_INFO'][:1000]
    pt.do_track_page_view(visited_url)



from django.conf import settings
from ipware.ip import get_ip
from server_side_matomo.tasks import record_analytic


class MatomoMiddleware(object):
    """ Record every request to piwik """
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)

        SITE_ID = getattr(settings, 'MATOMO_SITE_ID', None)
        if SITE_ID:
            ip = get_ip(request)
            keys_to_serialize = [
                'HTTP_USER_AGENT',
                'REMOTE_ADDR',
                'HTTP_REFERER',
                'HTTP_ACCEPT_LANGUAGE',
                'SERVER_NAME',
                'PATH_INFO',
                'QUERY_STRING',
            ]
            data = {
                'HTTPS': request.is_secure()        
            }
            for key in keys_to_serialize:
                if key in request.META:
                    data[key] = request.META[key]
            user_id = None
            if getattr(settings, 'MATOMO_TRACK_USER_ID', None) and request.user and request.user.pk:
                user_id = request.user.pk
            record_analytic.delay(data, ip, user_id)
        return response



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['server_side_matomo']

package_data = \
{'': ['*']}

install_requires = \
['Django>=1.11',
 'celery[redis]>=4.1.0',
 'django-ipware>=2.1.0',
 'piwikapi>=0.3']

setup_kwargs = {
    'name': 'django-server-side-matomo',
    'version': '2.0.2',
    'description': 'Send analytics data to Matomo using celery',
    'long_description': "Let your Django app perform server side analytics with Matomo. Server side analytics is a great way to get some analytics while respecting user privacy (only you see the data, no internet wide tracking) and performance (no js tracker needed!)\n\n# Quickstart\n\nYou'll need to have celery set up because making the Matomo request in your Django request would be really slow. This project will collect some data from a request using middleware, serialize it, and send it to celery. Works fine with the default celery json serializer. Running celery will not be described here.\n\n1. Install via pip `django-server-side-matomo`\n2. Add to INSTALLED_APPS `'server_side_matomo',`\n3. Add to MIDDLEWARE `'server_side_matomo.middleware.MatomoMiddleware'`\n4. Set the following in settings.py\n\n- MATOMO_SITE_ID = '1'  # Your site's Matomo ID\n- MATOMO_API_URL = 'https://your.site.com/piwik.php'\n- MATOMO_TOKEN_AUTH = 'your auth token'\n- MATOMO_TRACK_USER_ID = False  # Set to True to track user ID. See https://matomo.org/docs/user-id/\n\n# Testing and Development\n\nOnly merge requests with unit tests will be accepted. Please open an issue first if you'd like to propose a feature. I don't plan to add many features to this project myself. Unless other people are interested in doing the work - I have no plans to support things like Google Analytics.\n\n## Testing\n\nA Docker Compose file is provided to quickly try out the project. Just run in a web container:\n\n`./manage.py test`\n\nTested with Django 3.0 and Python 3.7.\n",
    'author': 'David Burke',
    'author_email': 'david@burkesoftware.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/burke-software/django-server-side-matomo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from django.test import TestCase
from unittest import mock
from piwikapi.tracking import PiwikTracker
from .tasks import record_analytic


class ServerSidePiwikTestCase(TestCase):
    @mock.patch('server_side_matomo.tasks.record_analytic.delay')
    def test_middleware(self, mocked):
        expected_headers = {'HTTPS': False, 'REMOTE_ADDR': '127.0.0.1', 'SERVER_NAME': 'testserver', 'PATH_INFO': '/admin/', 'QUERY_STRING': ''}
        expected_ip = '127.0.0.1'
        self.client.get('/admin/')
        mocked.assert_called_once_with(expected_headers, expected_ip, None)

    @mock.patch('piwikapi.tracking.PiwikTracker.do_track_page_view')
    def test_record_analytic(self, mocked):
        headers = {
            'HTTPS': False,
            'HTTP_USER_AGENT': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.94 Safari/537.36',
            'REMOTE_ADDR': '172.23.0.1',
            'HTTP_ACCEPT_LANGUAGE': 'en-US,en;q=0.9',
            'SERVER_NAME': 'a96cb4a08d4d',
            'PATH_INFO': '/admin/login/',
            'QUERY_STRING': 'next=/admin/',
        }
        ip = '172.23.0.1'
        record_analytic(headers, ip)
        mocked.assert_called_once_with('/admin/login/')
